// use this file to override specific colors in the application
// see https://github.com/archesproject/arches/tree/master/arches/app/media/css/themes/_default.scss 
// in the arches repo for the colors that can be overridden
//
// eg:
// $link-color: red;
