# Generated by Django 5.2.3 on 2025-06-16 15:09

import arches_modular_reports.models
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("arches_modular_reports", "0004_handle_for_concept_label_subregions"),
    ]

    operations = [
        migrations.AlterField(
            model_name="reportconfig",
            name="graph",
            field=models.ForeignKey(
                limit_choices_to=arches_modular_reports.models.get_graph_choices,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="report",
                to="models.graphmodel",
            ),
        ),
    ]
