# Generated by Django 5.2.5 on 2025-08-28 16:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("arches_modular_reports", "0006_improve_report_config_related_name"),
        ("models", "11499_add_editlog_resourceinstance_idx"),
    ]

    operations = [
        migrations.AddField(
            model_name="reportconfig",
            name="slug",
            field=models.TextField(default="default"),
        ),
        migrations.AddConstraint(
            model_name="reportconfig",
            constraint=models.UniqueConstraint(
                fields=("graph", "slug"), name="unique_slug_graph"
            ),
        ),
    ]
