// declare untyped modules that have been added to your project in `package.json`
// Module homepage on npmjs.com uses logos "TS" or "DT" to indicate if typed

import("@/arches/declarations.d.ts");

declare module "numeral";
