<div class="modular-report-mounting-point"></div>

<style>
    .modular-report-mounting-point {
        position: relative;
        height: inherit;
    }
    .sidenav {
        overflow: hidden !important;
    }
</style>
