{% extends "arches_urls.htm" %}

{% load static %}
{% load l10n %}
{% load i18n %}

{% block arches_urls %}
{{ block.super }}
<div class="arches-urls"
    modular_report_config = "{% url 'modular_report_config' %}"
    api_node_tile_data = '(resourceid) => { return "{% url "api_node_tile_data" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", resourceid)}'
    api_node_presentation = '(resourceid) => { return "{% url "api_node_presentation" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", resourceid)}'
    api_nodegroup_tile_data = '(resourceinstanceid, nodegroupalias) => { return "{% url "api_nodegroup_tile_data" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" "alias" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", resourceinstanceid).replace("alias", nodegroupalias)}'
    api_instance_permissions = "{% url 'api_instance_permissions' %}"
    api_modular_reports_resource = '(graphslug, resourceinstanceid) => { return "{% url "arches_querysets:api-resource" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" "bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", graphslug).replace("bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb", resourceinstanceid)}'
    api_modular_reports_tile = '(graphslug, nodegroupalias, tileid) => { return "{% url "arches_querysets:api-tile" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" "bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb" "cccccccc-cccc-cccc-cccc-cccccccccccc" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", graphslug).replace("bbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb", nodegroupalias).replace("cccccccc-cccc-cccc-cccc-cccccccccccc", tileid)}'
    api_has_permissions = "{% url 'api_has_permissions' %}"
    api_client_language_settings = "{% url 'api_client_language_settings' %}"
    api_related_resources = '(resourceinstanceid, relatedgraphslug) => { return "{% url "api_related_resources" "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa" "slug" %}".replace("aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa", resourceinstanceid).replace("slug", relatedgraphslug)}'
></div>
{% endblock arches_urls %}
